from .prompt import BASE_PROMPT, CHAT_PROMPT, CHAT_PROMPT_GENERAL
from .extract_key_prompt import EXTRACT_KEY_PROMPT

__all__ = [
    'BASE_PROMPT',
    'CHAT_PROMPT',
    'CHAT_PROMPT_GENERAL',
    'EXTRACT_KEY_PROMPT',
]
